#define CHIP_ID 0x1519
/******************************************************************************************************************/
/*                                                                                                                */
/*                                              direct addressable registers                                      */
/*                                                                                                                */
/******************************************************************************************************************/

//        NAME                          page,address
#define CHIP_VERSION                       0,0x00
#define ROW_SPEED                          0,0x0A
#define CLOCK                              0,0x65
#define CONTEXT_CONTROL                    0,0xF2

#define OUTPUT_FORMAT                      1,0x97

// JPEG related
#define JPEG_CONTROL_REGISTER              2,0x00
#define JPEG_STATUS_REGISTER_0             2,0x02
#define JPEG_STATUS_REGISTER_1             2,0x03
#define JPEG_STATUS_REGISTER_2             2,0x04
#define JPEG_FRONTEND_CONFIG_REGISTER      2,0x05
#define JPEG_CORE_CONFIG_REGISTER          2,0x06
#define JPEG_ENCODER_BYPASS                2,0x0A

#define OUTPUT_CONFIGURATION               2,0x0D  // set to 0x58 for JPEG SOI+EOI within frame valid + variable pixel clock rate + jpeg status insertion at end of frame
#define PCLK1_PCLK2_CONFIGURATION          2,0x0E  // set to 0xF0F for max divider and 0 slew rate
#define PCLK3_CONFIGURATION                2,0x0F  // set to 0xF for max divider and 0 slew rate

/******************************************************************************************************************/
/*                                                                                                                */
/*                                              Firmware / MCU registers                                          */
/*                                                                                                                */
/******************************************************************************************************************/

/*

R0xC6:1[15:0], Indirect Access Address Register
Bit Description
7:0 Bits            7:0   of address for physical access; driver variable offset for logical access.
12:8 Bits           12:8  of address for physical access; driver ID for logical access.
14:13 Bits          14:13 of address for physical access; R0xC6:1[14:13] = 01 select logical access.
15 1 = 8-bit access; 0 = 16-bit access
*/

#define MON_VER                     0xA00C // mon.ver (ID = 0, Offset = 12, 8-bit variable) // firmware version #


/*
To switch from preview to the capture state, set the following variables:
&#65533; seq.captureParams.mode[1] = 1// ID = 1, Offset = 0x20
&#65533; seq.cmd = 2// ID = 1, Offset = 0x03
To switch from capture back into preview mode, use the following settings:
&#65533; seq.captureParams.mode[1] = 0// ID = 1, Offset = 0x20
&#65533; seq.cmd = 1// ID = 1, Offset = 0x03
*/
#define VMT                         0x2000 // state
#define SEQ_CAPTUREPARAMS_MODE      0xA120 // seq.captureParams.mode[1] = 1// ID = 1, Offset = 0x20
#define SEQ_CAPTUREPARAMS_NUMFRAMES 0xA121 // seq.captureParams.numFrames  (ID = 1, Offset = 0x21)
#define SEQ_CMD                     0xA103 // seq.cmd = 1// ID = 1, Offset = 0x03
#define SEQ_STATE                   0xA104 // state






/*
The current context mode can also be read through the serial interface from the variable
mode.context (DriverID = 7, Offset = 0x02). If mode.context is read back as logic "1",
the capture mode is active. When the bit is cleared, the current mode is preview.
*/
#define MODE_CONTEXT            0xA702


//JPEG enable: mode.mode_config[5] = 0 // ID = 7, offset = 0x0B
#define MODE_MODE_CONFIG        0x270B

/*
To change the output size of a context, the associated context image height and width
values can be updated before switching to the targeted context. If the size is changed for
the current (active) context, then a refresh command (seq.cmd = 5) needs to be
executed additionally.
For context A, use the variable mode.Output Width A (ID = 7, offset = 3) and
mode.Output Height A (ID = 7, offset = 5).
For context B, use the variable mode.Output Width B (ID = 7, offset = 7) and
mode.Output Height B (ID = 7, offset = 9). These settings only change the output image
size and do not have any effect on the frame rate or field of view.

*/
#define MODE_OUTPUT_HEIGHT_A    0x2705
#define MODE_OUTPUT_WIDTH_A     0x2703
#define MODE_OUTPUT_HEIGHT_B    0x2709
#define MODE_OUTPUT_WIDTH_B     0x2707


/*
mode.crop_X0_B// ID = 7, Offset = 0x35
mode.crop_X1_B// ID = 7, Offset = 0x37
mode.crop_Y0_B// ID = 7, Offset = 0x39
mode.crop_Y1_B// ID = 7, Offset = 0x3B
*/
#define MODE_CROP_X0_B          0x2735
#define MODE_CROP_Y0_B          0x2739
#define MODE_CROP_X1_B          0x2737
#define MODE_CROP_Y1_B          0x273B

/*
To select between YUV and RGB, bit 5 of variables mode.output_format_A (ID = 7,
Offset = 0x7D) and mode.output_format_B (ID = 7, Offset = 0x7E) should be used,
depending on which context mode is of interest.
*/
#define MODE_OUTPUT_FORMAT_A    0xA77D
#define MODE_OUTPUT_FORMAT_B    0xA77E

#define MODE_FIFO_CONFIG0_B     0x2772

/* JPEG 
0 vmt void* 58451 RW Reserved.
2 width uint 1600 R Image width (see mode.output_width).
4 height uint 1200 R Image height (see mode.output_height).
6 format uchar 0 RW Image format:
0&#65533;YCbCr 4:2:2
1&#65533;YCbCr 4:2:0
2&#65533;Monochrome
7 config uchar 52 RW Configuration and handshaking:
Bit 0&#65533;if 1, video; if 0, still snapshot
Bit 1&#65533;enable handshaking with host at every error frame
Bit 2&#65533;enable retry after an unsuccessful encode or transfer
Bit 3&#65533;host indicates it is ready for next frame
Bit 4&#65533;enable scaled quantization table generation
Bit 5&#65533;enable auto-select quantization table
Bit 7:6&#65533;quantization table ID
8 restartInt uint 0 RW Restart marker interval:
0 = restart marker is disabled
10 qscale1 uchar 6 RWBit 6:0&#65533;scaling factor for first set of quantization tables
Bit 7&#65533;1, new scaling factor value
11 qscale2 uchar 9 RWBit 6:0&#65533;scaling factor for second set of quantization tables
Bit 7&#65533;if 1, new scaling factor value
12 qscale3 uchar 12 RW Bit 6:0&#65533;scaling factor for third set of quantization tables
Bit 7&#65533;if 1, new scaling factor value
13 timeoutFrames uchar 10 RW Number of frames to time out when host is not responding
(setting bit 3 of config) to an unsuccessful JPEG frame while bit
1 of config is set.
14 state uchar 0 R JPEG driver state.
15 dataLengthMSB uchar R Bit [23:16] of previous frame JPEG data length.
16 dataLengthLSBs uint R Bit [15:0] of previous frame JPEG data length.
*/

#define JPEG_STATE                             0xA90E
#define JPEG_DATALENGTH_MSB                    0xA90F
#define JPEG_DATALENGTH_LSB                    0xA910


/****************************** MACROS ******************************************************/



/******************************************** FUNCTIONS *************************************/
void mt9d11_init();
void mt9d111_frequency(int freq);
bool writeRegister(int page, unsigned char address, unsigned short value, unsigned short mask = 0xFFFF);
bool readRegister(int page, unsigned char address, char *value);
unsigned short usReadRegister(int page,unsigned char address);
void pageChange(int page);
unsigned short usReadRegister(int page, unsigned char address);
bool writeMCUMemory(unsigned short address, unsigned short value, unsigned short mask = 0xFFFF);
bool readMCUMemory(unsigned short address, char *value);
unsigned short readMCU(unsigned short address);
void verifyMCUMemory(unsigned short address, unsigned short value);
void printMCU(unsigned short address, char *name);
void verifyRegister(int page, unsigned char address, unsigned short value, unsigned short mask = 0xFFFF);
void printRegister(int page, unsigned short address, char* name);
